@@ -2,7 +2,7 @@
 #'
 #' Wrapper function for NNS nowcasting method using \link{NNS.VAR} as detailed in Viole (2020), \url{https://www.ssrn.com/abstract=3586658}.
 #'
-#' @param h integer; \code{(h = 12)} (default) Number of periods to forecast. \code{(h = 0)} will return just the interpolated and extrapolated values.
+#' @param h integer; \code{(h = 1)} (default) Number of periods to forecast. \code{(h = 0)} will return just the interpolated and extrapolated values.
 #' @param additional.regressors character; \code{NULL} (default) add more regressors to the base model.  The format must utilize the \code{\link[quantmod]{getSymbols}} format for FRED data.
 #' @param specific.regressors integer; \code{NULL} (default) Select individual regressors from the base model per Viole (2020) listed in the References.
 #' @param start.date character; \code{"2000-01-03"} (default) Starting date for all data series download.
@@ -43,7 +43,7 @@
 #' @export
 
 
-NNS.nowcast <- function(h = 12,
+NNS.nowcast <- function(h = 1,
                         additional.regressors = NULL,
                         specific.regressors = NULL,
                         start.date = "2000-01-03",
@@ -57,11 +57,10 @@ NNS.nowcast <- function(h = 12,
                  "BOPTIMP", "TTLCONS", "IR", "CPILFESL", "PCEPILFE",
                  "PCEPI", "PERMIT", "TCU", "BUSINV", "ULCNFB",
                  "IQ", "GACDISA066MSFRBNY", "GACDFSA066MSFRBPHI", "PCEC96", "GDPC1",
-                 "ICSA", "DGS10", "T10Y2Y", "ICSA", "WALCL")
+                 "ICSA", "DGS10", "T10Y2Y", "WALCL")
 
-  variable_list <- character()
 
-  if(is.null(specific.regressors)) variable_list <- c(variables, additional.regressors) else variable_list <- c(variables, additional.regressors)[specific.regressors]
+  if(is.null(specific.regressors)) variable_list <- as.character(c(variables, additional.regressors)) else variable_list <- as.character(c(variables, additional.regressors)[specific.regressors])
 
   NNSdata <- new.env()
   quantmod::getSymbols(variable_list, src = "FRED", env = NNSdata)
@@ -70,12 +69,18 @@ NNS.nowcast <- function(h = 12,
     quantmod::getSymbols(variable_list[!variable_list%in%ls(envir = NNSdata)[(ls(envir = NNSdata)%in%variable_list)]], src = "FRED")
   }
  
-  raw_econ_variables <- lapply(mget(variable_list, envir = NNSdata), function(x) xts::to.monthly(x)[,1])
+  raw_econ_variables <- lapply(mget(variable_list, envir = NNSdata), function(x) xts::to.monthly(x)[,4])
+  
+  oldw <- getOption("warn")
+  options(warn = -1)
+  
   if(!keep.data) rm(list = ls(), envir = NNSdata)
   
   econ_variables <- Reduce(function(...) merge(..., all=TRUE), raw_econ_variables)[paste0(start.date,"::")]
-  
-  colnames(econ_variables) <- variable_list
 
+  colnames(econ_variables) <- variable_list
+  
+  options(warn = oldw)
+  
   NNS.VAR(econ_variables, h = h, tau = 12, status = status, ncores = ncores, nowcast = TRUE)
 }
